@@ -4,7 +4,7 @@ defmodule List.Chars.AtomTest do
   use ExUnit.Case, async: true
 
   test "basic" do
-    assert to_char_list(:foo) == 'foo'
+    assert to_charlist(:foo) == 'foo'
   end
 end
 
@@ -12,7 +12,7 @@ defmodule List.Chars.BitStringTest do
   use ExUnit.Case, async: true
 
   test "basic" do
-    assert to_char_list("foo") == 'foo'
+    assert to_charlist("foo") == 'foo'
   end
 end
 
@@ -20,11 +20,11 @@ defmodule List.Chars.NumberTest do
   use ExUnit.Case, async: true
 
   test "integer" do
-    assert to_char_list(1) == '1'
+    assert to_charlist(1) == '1'
   end
 
   test "float" do
-    assert to_char_list(1.0) == '1.0'
+    assert to_charlist(1.0) == '1.0'
   end
 end
 
@@ -32,6 +32,6 @@ defmodule List.Chars.ListTest do
   use ExUnit.Case, async: true
 
   test "basic" do
-    assert to_char_list([ 1, "b", 3 ]) == [1, "b", 3]
+    assert to_charlist([ 1, "b", 3 ]) == [1, "b", 3]
   end
 end
