@@ -18,6 +18,10 @@ defmodule RangeTest do
   test "range?" do
     assert Range.range?(1..3)
     refute Range.range?(0)
+    assert %Range{first: -10, last: 10} |> Range.range?
+    refute %Range{first: nil, last: 10} |> Range.range?
+    refute %Range{first: -10, last: nil} |> Range.range?
+    refute %Range{} |> Range.range?
   end
 
   test "enum" do
