@@ -12,7 +12,7 @@ defmodule StringTest do
   end
 
   # test cases described in http://mortoray.com/2013/11/27/the-string-type-is-broken/
-  test "unicode" do
+  test "Unicode" do
     assert String.reverse("noël") == "lëon"
     assert String.slice("noël", 0..2) == "noë"
     assert String.length("noël") == 4
@@ -66,6 +66,9 @@ defmodule StringTest do
     assert String.split("abé", "", trim: true, parts: 2) == ["a", "bé"]
 
     assert String.split("noël", "") == ["n", "o", "ë", "l", ""]
+
+    assert String.split("x-", "-", parts: 2, trim: true) == ["x"]
+    assert String.split("x-x-", "-", parts: 3, trim: true) == ["x", "x"]
   end
 
   test "split with regex" do
@@ -79,6 +82,14 @@ defmodule StringTest do
     assert String.split("a,b", ~r{\.}) == ["a,b"]
   end
 
+  test "split with compiled pattern" do
+    pattern = :binary.compile_pattern("-")
+
+    assert String.split("x-", pattern) == ["x", ""]
+    assert String.split("x-", pattern, parts: 2, trim: true) == ["x"]
+    assert String.split("x-x-", pattern, parts: 3, trim: true) == ["x", "x"]
+  end
+
   test "splitter" do
     assert String.splitter("a,b,c", ",") |> Enum.to_list == ["a", "b", "c"]
     assert String.splitter("a,b", ".") |> Enum.to_list == ["a,b"]
@@ -194,78 +205,123 @@ defmodule StringTest do
     assert String.replace_trailing("aaa", "b", "c") == "aaa"
   end
 
-  test "rstrip" do
-    assert String.rstrip("") == ""
-    assert String.rstrip("1\n") == "1"
-    assert String.rstrip("\r\n") == ""
-    assert String.rstrip("   abc  ") == "   abc"
-    assert String.rstrip("   abc a") == "   abc a"
-    assert String.rstrip("a  abc  a\n\n") == "a  abc  a"
-    assert String.rstrip("a  abc  a\t\n\v\f\r\s") == "a  abc  a"
-    assert String.rstrip("a  abc  a" <> <<194, 133>>) == "a  abc  a"
-    assert String.rstrip("   abc aa", ?a) == "   abc "
-    assert String.rstrip("   abc __", ?_) == "   abc "
-    assert String.rstrip(" aaaaaaaaa", ?a) == " "
-    assert String.rstrip("aaaaaaaaaa", ?a) == ""
-    assert String.rstrip("]]]]]]]]]]", ?]) == ""
-    assert String.rstrip("   cat 猫猫", ?猫) == "   cat "
-    # information separators are not whitespace
-    assert String.rstrip("a  abc  a \u001F") == "a  abc  a \u001F"
-    # no-break space
-    assert String.rstrip("a  abc  a \u00A0") == "a  abc  a"
-  end
+  test "trim" do
+    assert String.trim("") == ""
+    assert String.trim("  abc ") == "abc"
+    assert String.trim("a  abc  a\n\n") == "a  abc  a"
+    assert String.trim("a  abc  a\t\n\v\f\r\s") == "a  abc  a"
 
-  test "lstrip" do
-    assert String.lstrip("") == ""
-    assert String.lstrip("   abc  ") == "abc  "
-    assert String.lstrip("a  abc  a") == "a  abc  a"
-    assert String.lstrip("\n\na  abc  a") == "a  abc  a"
-    assert String.lstrip("\t\n\v\f\r\sa  abc  a") == "a  abc  a"
-    assert String.lstrip(<<194, 133>> <> "a  abc  a") == "a  abc  a"
-    assert String.lstrip("__  abc  _", ?_) == "  abc  _"
-    assert String.lstrip("猫猫 cat   ", ?猫) == " cat   "
-    # information separators are not whitespace
-    assert String.lstrip("\u001F a  abc  a") == <<31>> <> " a  abc  a"
+    assert String.trim("___  abc  ___", "_") == "  abc  "
+    assert String.trim("猫猫猫cat猫猫猫", "猫猫") == "猫cat猫"
     # no-break space
-    assert String.lstrip("\u00A0 a  abc  a") == "a  abc  a"
-  end
-
-  test "strip" do
-    assert String.strip("") == ""
-    assert String.strip("   abc  ") == "abc"
-    assert String.strip("a  abc  a\n\n") == "a  abc  a"
-    assert String.strip("a  abc  a\t\n\v\f\r\s") == "a  abc  a"
-    assert String.strip("___  abc  ___", ?_) == "  abc  "
-    assert String.strip("猫猫猫  cat  猫猫猫", ?猫) == "  cat  "
-    # no-break space
-    assert String.strip("\u00A0a  abc  a\u00A0") == "a  abc  a"
+    assert String.trim("\u00A0a  abc  a\u00A0") == "a  abc  a"
     # whitespace defined as a range
-    assert String.strip("\u2008a  abc  a\u2005") == "a  abc  a"
+    assert String.trim("\u2008a  abc  a\u2005") == "a  abc  a"
   end
 
-  test "rjust" do
-    assert String.rjust("", 5) == "     "
-    assert String.rjust("abc", 5) == "  abc"
-    assert String.rjust("  abc  ", 9) == "    abc  "
-    assert String.rjust("猫", 5) == "    猫"
-    assert String.rjust("abc", 5, ?-) == "--abc"
-    assert String.rjust("abc", 5, ?猫) == "猫猫abc"
-    assert String.rjust("-", 0) == "-"
+  test "trim_leading" do
+    assert String.trim_leading("") == ""
+    assert String.trim_leading("   abc  ") == "abc  "
+    assert String.trim_leading("a  abc  a") == "a  abc  a"
+    assert String.trim_leading("\n\na  abc  a") == "a  abc  a"
+    assert String.trim_leading("\t\n\v\f\r\sa  abc  a") == "a  abc  a"
+    assert String.trim_leading(<<194, 133, "a  abc  a">>) == "a  abc  a"
+    # information separators are not whitespace
+    assert String.trim_leading("\u001F a  abc  a") == "\u001F a  abc  a"
+    # no-break space
+    assert String.trim_leading("\u00A0 a  abc  a") == "a  abc  a"
+
+    assert String.trim_leading("aa aaa", "aaa") == "aa aaa"
+    assert String.trim_leading("aaa aaa", "aa") == "a aaa"
+    assert String.trim_leading("aa abc   ", "a") == " abc   "
+    assert String.trim_leading("__ abc   ", "_") == " abc   "
+    assert String.trim_leading("aaaaaaaaa ", "a") == " "
+    assert String.trim_leading("aaaaaaaaaa", "a") == ""
+    assert String.trim_leading("]]]]]] ]", "]") == " ]"
+    assert String.trim_leading("猫猫 cat   ", "猫") == " cat   "
+    assert String.trim_leading("test", "t") == "est"
+    assert String.trim_leading("t", "t") == ""
+    assert String.trim_leading("", "t") == ""
+  end
+
+  test "trim_trailing" do
+    assert String.trim_trailing("") == ""
+    assert String.trim_trailing("1\n") == "1"
+    assert String.trim_trailing("\r\n") == ""
+    assert String.trim_trailing("   abc  ") == "   abc"
+    assert String.trim_trailing("   abc a") == "   abc a"
+    assert String.trim_trailing("a  abc  a\n\n") == "a  abc  a"
+    assert String.trim_trailing("a  abc  a\t\n\v\f\r\s") == "a  abc  a"
+    assert String.trim_trailing(<<"a  abc  a", 194, 133>>) == "a  abc  a"
+    # information separators are not whitespace
+    assert String.trim_trailing("a  abc  a \u001F") == "a  abc  a \u001F"
+    # no-break space
+    assert String.trim_trailing("a  abc  a \u00A0") == "a  abc  a"
+
+    assert String.trim_trailing("aaa aa", "aaa") == "aaa aa"
+    assert String.trim_trailing("aaa aaa", "aa") == "aaa a"
+    assert String.trim_trailing("   abc aa", "a") == "   abc "
+    assert String.trim_trailing("   abc __", "_") == "   abc "
+    assert String.trim_trailing(" aaaaaaaaa", "a") == " "
+    assert String.trim_trailing("aaaaaaaaaa", "a") == ""
+    assert String.trim_trailing("] ]]]]]]", "]") == "] "
+    assert String.trim_trailing("   cat 猫猫", "猫") == "   cat "
+    assert String.trim_trailing("test", "t") == "tes"
+    assert String.trim_trailing("t", "t") == ""
+    assert String.trim_trailing("", "t") == ""
+  end
+
+  test "pad_leading" do
+    assert String.pad_leading("", 5) == "     "
+    assert String.pad_leading("abc", 5) == "  abc"
+    assert String.pad_leading("  abc  ", 9) == "    abc  "
+    assert String.pad_leading("猫", 5) == "    猫"
+    assert String.pad_leading("-", 0) == "-"
+    assert String.pad_leading("-", 1) == "-"
+
+    assert String.pad_leading("---", 5, "abc") == "ab---"
+    assert String.pad_leading("---", 9, "abc") == "abcabc---"
+
+    assert String.pad_leading("---", 5, ["abc"]) == "abcabc---"
+    assert String.pad_leading("--", 6, ["a", "bc"]) == "abcabc--"
+
     assert_raise FunctionClauseError, fn ->
-      String.rjust("-", -1)
+      String.pad_leading("-", -1)
+    end
+    assert_raise FunctionClauseError, fn ->
+      String.pad_leading("-", 1, [])
+    end
+
+    message = "expected a string padding element, got: 10"
+    assert_raise ArgumentError, message, fn ->
+      String.pad_leading("-", 3, ["-", 10])
     end
   end
 
-  test "ljust" do
-    assert String.ljust("", 5) == "     "
-    assert String.ljust("abc", 5) == "abc  "
-    assert String.ljust("  abc  ", 9) == "  abc    "
-    assert String.ljust("猫", 5) == "猫    "
-    assert String.ljust("abc", 5, ?-) == "abc--"
-    assert String.ljust("abc", 5, ?猫) == "abc猫猫"
-    assert String.ljust("-", 0) == "-"
+  test "pad_trailing" do
+    assert String.pad_trailing("", 5) == "     "
+    assert String.pad_trailing("abc", 5) == "abc  "
+    assert String.pad_trailing("  abc  ", 9) == "  abc    "
+    assert String.pad_trailing("猫", 5) == "猫    "
+    assert String.pad_trailing("-", 0) == "-"
+    assert String.pad_trailing("-", 1) == "-"
+
+    assert String.pad_trailing("---", 5, "abc") == "---ab"
+    assert String.pad_trailing("---", 9, "abc") == "---abcabc"
+
+    assert String.pad_trailing("---", 5, ["abc"]) == "---abcabc"
+    assert String.pad_trailing("--", 6, ["a", "bc"]) == "--abcabc"
+
     assert_raise FunctionClauseError, fn ->
-      String.ljust("-", -1)
+      String.pad_trailing("-", -1)
+    end
+    assert_raise FunctionClauseError, fn ->
+      String.pad_trailing("-", 1, [])
+    end
+
+    message = "expected a string padding element, got: 10"
+    assert_raise ArgumentError, message, fn ->
+      String.pad_trailing("-", 3, ["-", 10])
     end
   end
 
@@ -497,17 +553,8 @@ defmodule StringTest do
     assert String.valid?("øsdfh")
     assert String.valid?("dskfjあska")
 
-    refute String.valid?(<<0xffff :: 16>>)
-    refute String.valid?("asd" <> <<0xffff :: 16>>)
-  end
-
-  test "valid_character?" do
-    assert String.valid_character?("a")
-    assert String.valid_character?("ø")
-    assert String.valid_character?("あ")
-
-    refute String.valid_character?("\uFFFF")
-    refute String.valid_character?("ab")
+    refute String.valid?(<<0xFFFF :: 16>>)
+    refute String.valid?("asd" <> <<0xFFFF :: 16>>)
   end
 
   test "chunk valid" do
@@ -516,11 +563,11 @@ defmodule StringTest do
     assert String.chunk("ødskfjあ\x11ska", :valid)
            == ["ødskfjあ\x11ska"]
     assert String.chunk("abc\u{0ffff}def", :valid)
-           == ["abc", <<0x0ffff::utf8>>, "def"]
-    assert String.chunk("\u{0fffe}\u{3ffff}привет\u{0ffff}мир", :valid)
-           == [<<0x0fffe::utf8, 0x3ffff::utf8>>, "привет", <<0x0ffff::utf8>>, "мир"]
-    assert String.chunk("日本\u{0ffff}\u{fdef}ござございます\u{fdd0}", :valid)
-           == ["日本", <<0x0ffff::utf8, 0xfdef::utf8>>, "ござございます", <<0xfdd0::utf8>>]
+           == ["abc", <<0x0FFFF::utf8>>, "def"]
+    assert String.chunk("\u{0FFFE}\u{3FFFF}привет\u{0FFFF}мир", :valid)
+           == [<<0x0FFFE::utf8, 0x3FFFF::utf8>>, "привет", <<0x0FFFF::utf8>>, "мир"]
+    assert String.chunk("日本\u{0FFFF}\u{FDEF}ござございます\u{FDD0}", :valid)
+           == ["日本", <<0x0FFFF::utf8, 0xFDEF::utf8>>, "ござございます", <<0xFDD0::utf8>>]
   end
 
   test "chunk printable" do
@@ -528,8 +575,8 @@ defmodule StringTest do
 
     assert String.chunk("ødskfjあska", :printable)
            == ["ødskfjあska"]
-    assert String.chunk("abc\x{0ffff}def", :printable)
-           == ["abc", <<0x0ffff::utf8>>, "def"]
+    assert String.chunk("abc\u{0FFFF}def", :printable)
+           == ["abc", <<0x0FFFF::utf8>>, "def"]
     assert String.chunk("\x06ab\x05cdef\x03\0", :printable)
            == [<<6>>, "ab", <<5>>, "cdef", <<3, 0>>]
   end
@@ -569,18 +616,18 @@ defmodule StringTest do
     refute String.contains? "elixir of life", ["death", "mercury", "eternal life"]
   end
 
-  test "to char list" do
-    assert String.to_char_list("æß")  == [?æ, ?ß]
-    assert String.to_char_list("abc") == [?a, ?b, ?c]
+  test "to charlist" do
+    assert String.to_charlist("æß")  == [?æ, ?ß]
+    assert String.to_charlist("abc") == [?a, ?b, ?c]
 
     assert_raise UnicodeConversionError,
                  "invalid encoding starting at <<223, 255>>", fn ->
-      String.to_char_list(<< 0xDF, 0xFF >>)
+      String.to_charlist(<< 0xDF, 0xFF >>)
     end
 
     assert_raise UnicodeConversionError,
                  "incomplete encoding starting at <<195>>", fn ->
-      String.to_char_list(<< 106, 111, 115, 195 >>)
+      String.to_charlist(<< 106, 111, 115, 195 >>)
     end
   end
 
@@ -616,5 +663,15 @@ defmodule StringTest do
     assert String.jaro_distance("sean", "susan") == 0.7833333333333333
     assert String.jaro_distance("jon", "john") == 0.9166666666666666
     assert String.jaro_distance("jon", "jan") == 0.7777777777777777
+    assert String.jaro_distance("семена", "стремя") == 0.6666666666666666
+  end
+
+  test "difference/2" do
+    assert String.myers_difference("", "abc") == [ins: "abc"]
+    assert String.myers_difference("abc", "") == [del: "abc"]
+    assert String.myers_difference("", "") == []
+    assert String.myers_difference("abc", "abc") == [eq: "abc"]
+    assert String.myers_difference("abc", "aйbc") == [eq: "a", ins: "й", eq: "bc"]
+    assert String.myers_difference("aйbc", "abc") == [eq: "a", del: "й", eq: "bc"]
   end
 end
