@@ -16,6 +16,13 @@ defmodule ExUnit.AssertionError do
   def no_value do
     @no_value
   end
+
+  def message(exception) do
+    "\n\n" <>
+      ExUnit.Formatter.format_assertion_error(exception, :infinity, &formatter/2, "")
+  end
+
+  defp formatter(_, msg), do: msg
 end
 
 defmodule ExUnit.MultiError do
@@ -26,8 +33,10 @@ defmodule ExUnit.MultiError do
   defexception [errors: []]
 
   def message(exception) do
-    "got the following errors:\n\n  * " <>
-      Enum.map_join(exception, "\n  * ", &Exception.message/1)
+    "got the following errors:\n\n" <>
+      Enum.map_join(exception, "\n\n", fn {kind, error, stack} ->
+        Exception.format_banner(kind, error, stack)
+      end)
   end
 end
 
@@ -97,6 +106,7 @@ defmodule ExUnit.Assertions do
 
     left = Macro.expand(left, __CALLER__)
     vars = collect_vars_from_pattern(left)
+    pins = collect_pins_from_pattern(left)
 
     # If the match works, we need to check if the value
     # is not nil nor false. We need to rewrite the if
@@ -125,7 +135,8 @@ defmodule ExUnit.Assertions do
             raise ExUnit.AssertionError,
               right: right,
               expr: expr,
-              message: "match (=) failed"
+              message: "match (=) failed" <>
+                       ExUnit.Assertions.__pins__(unquote(pins))
         end
       right
     end
@@ -134,12 +145,15 @@ defmodule ExUnit.Assertions do
   defmacro assert({:match?, meta, [left, right]} = assertion) do
     code   = Macro.escape(assertion)
     match? = {:match?, meta, [left, Macro.var(:right, __MODULE__)]}
+    pins   = collect_pins_from_pattern(left)
+
     quote do
       right = unquote(right)
       assert unquote(match?),
         right: right,
         expr: unquote(code),
-        message: "match (match?) failed"
+        message: "match (match?) failed" <>
+                 ExUnit.Assertions.__pins__(unquote(pins))
     end
   end
 
@@ -187,12 +201,15 @@ defmodule ExUnit.Assertions do
   defmacro refute({:match?, meta, [left, right]} = assertion) do
     code   = Macro.escape(assertion)
     match? = {:match?, meta, [left, Macro.var(:right, __MODULE__)]}
+    pins   = collect_pins_from_pattern(left)
+
     quote do
       right = unquote(right)
       refute unquote(match?),
         right: right,
         expr: unquote(code),
-        message: "match (match?) succeeded, but should have failed"
+        message: "match (match?) succeeded, but should have failed" <>
+                 ExUnit.Assertions.__pins__(unquote(pins))
     end
   end
 
@@ -272,12 +289,13 @@ defmodule ExUnit.Assertions do
   end
 
   @doc """
-  Asserts a message was or is going to be received.
+  Asserts that a message matching `pattern` was or is going to be received.
 
   Unlike `assert_received`, it has a default timeout
   of 100 milliseconds.
 
-  The `expected` argument is a pattern.
+  The `pattern` argument must be a match pattern. Flunks with `failure_message`
+  if a message matching `pattern` is not received.
 
   ## Examples
 
@@ -295,43 +313,52 @@ defmodule ExUnit.Assertions do
       assert_receive {:count, ^x}
 
   """
-  defmacro assert_receive(expected,
+  defmacro assert_receive(pattern,
                           timeout \\ Application.fetch_env!(:ex_unit, :assert_receive_timeout),
-                          message \\ nil) do
-    do_assert_receive(expected, timeout, message, __CALLER__)
+                          failure_message \\ nil) do
+    do_assert_receive(pattern, timeout, failure_message, __CALLER__)
   end
 
   @doc """
-  Asserts a message was received and is in the current process' mailbox.
-  Timeout is set to 0, so there is no waiting time.
+  Asserts that a message matching `pattern` was received and is in the
+  current process' mailbox.
 
-  The `expected` argument is a pattern.
+  The `pattern` argument must be a match pattern. Flunks with `failure_message`
+  if a message matching `pattern` was not received.
+
+  Timeout is set to 0, so there is no waiting time.
 
   ## Examples
 
       send self, :hello
       assert_received :hello
 
+      send self, :bye
+      assert_received :hello, "Oh No!"
+      ** (ExUnit.AssertionError) Oh No!
+      Process mailbox:
+        :bye
+
   You can also match against specific patterns:
 
       send self, {:hello, "world"}
       assert_received {:hello, _}
 
   """
-  defmacro assert_received(expected, message \\ nil) do
-    do_assert_receive(expected, 0, message, __CALLER__)
+  defmacro assert_received(pattern, failure_message \\ nil) do
+    do_assert_receive(pattern, 0, failure_message, __CALLER__)
   end
 
-  defp do_assert_receive(expected, timeout, message, caller) do
-    binary = Macro.to_string(expected)
+  defp do_assert_receive(pattern, timeout, failure_message, caller) do
+    binary = Macro.to_string(pattern)
 
     # Expand before extracting metadata
-    expected = Macro.expand(expected, caller)
-    vars = collect_vars_from_pattern(expected)
-    pins = collect_pins_from_pattern(expected)
+    pattern = Macro.expand(pattern, caller)
+    vars = collect_vars_from_pattern(pattern)
+    pins = collect_pins_from_pattern(pattern)
 
     pattern =
-      case expected do
+      case pattern do
         {:when, meta, [left, right]} ->
           {:when, meta, [quote(do: unquote(left) = received), right]}
         left ->
@@ -347,12 +374,11 @@ defmodule ExUnit.Assertions do
             {received, unquote(vars)}
         after
           timeout ->
-            message = unquote(message) || "No message matching #{unquote(binary)} after #{timeout}ms."
-            flunk(message <> ExUnit.Assertions.__pins__(unquote(pins))
+            failure_message = unquote(failure_message) || "No message matching #{unquote(binary)} after #{timeout}ms."
+            flunk(failure_message <> ExUnit.Assertions.__pins__(unquote(pins))
                           <> ExUnit.Assertions.__mailbox__(self()))
         end
 
-      _ = unquote(vars) # Silence warnings
       received
     end
   end
@@ -394,13 +420,21 @@ defmodule ExUnit.Assertions do
     {_, pins} =
       Macro.prewalk(expr, [], fn
         {:^, _, [{name, _, _} = var]}, acc ->
-          {:ok, [{name, var}|acc]}
+          {:ok, [{name, var} | acc]}
         form, acc ->
           {form, acc}
       end)
     Enum.uniq_by(pins, &elem(&1, 0))
   end
 
+  defp collect_vars_from_pattern({:when, _, [left, right]}) do
+    pattern = collect_vars_from_pattern(left)
+    for {name, _, context} = var <- collect_vars_from_pattern(right),
+      Enum.any?(pattern, &match?({^name, _, ^context}, &1)),
+      into: pattern,
+      do: var
+  end
+
   defp collect_vars_from_pattern(expr) do
     Macro.prewalk(expr, [], fn
       {:::, _, [left, _]}, acc ->
@@ -435,8 +469,8 @@ defmodule ExUnit.Assertions do
         1 + "test"
       end
 
-      assert_raise RuntimeError, ~r/^Today's lucky number is 0\.\d+!$/, fn ->
-        raise "Today's lucky number is #{:rand.uniform}!"
+      assert_raise RuntimeError, ~r/^today's lucky number is 0\.\d+!$/, fn ->
+        raise "today's lucky number is #{:rand.uniform}!"
       end
   """
   def assert_raise(exception, message, function) when is_function(function) do
@@ -573,10 +607,11 @@ defmodule ExUnit.Assertions do
   end
 
   @doc """
-  Asserts `message` was not received (and won't be received) within
-  the `timeout` period.
+  Asserts that a message matching `pattern` was not received (and won't be received)
+  within the `timeout` period.
 
-  The `not_expected` argument is a match pattern.
+  The `pattern` argument must be a match pattern. Flunks with `failure_message`
+  if a message matching `pattern` is received.
 
   ## Examples
 
@@ -587,15 +622,18 @@ defmodule ExUnit.Assertions do
       refute_receive :bye, 1000
 
   """
-  defmacro refute_receive(not_expected,
+  defmacro refute_receive(pattern,
                           timeout \\ Application.fetch_env!(:ex_unit, :refute_receive_timeout),
-                          message \\ nil) do
-    do_refute_receive(not_expected, timeout, message)
+                          failure_message \\ nil) do
+    do_refute_receive(pattern, timeout, failure_message)
   end
 
   @doc """
-  Asserts a message was not received (i.e. it is not in the current process mailbox).
-  The `not_expected` argument must be a match pattern.
+  Asserts a message matching `pattern` was not received (i.e. it is not in the
+  current process' mailbox).
+
+  The `pattern` argument must be a match pattern. Flunks with `failure_message`
+  if a message matching `pattern` was received.
 
   Timeout is set to 0, so there is no waiting time.
 
@@ -604,13 +642,19 @@ defmodule ExUnit.Assertions do
       send self, :hello
       refute_received :bye
 
+      send self, :hello
+      refute_received :hello, "Oh No!"
+      ** (ExUnit.AssertionError) Oh No!
+      Process mailbox:
+        :bye
+
   """
-  defmacro refute_received(not_expected, message \\ nil) do
-    do_refute_receive(not_expected, 0, message)
+  defmacro refute_received(pattern, failure_message \\ nil) do
+    do_refute_receive(pattern, 0, failure_message)
   end
 
-  defp do_refute_receive(not_expected, timeout, message) do
-    receive_clause = refute_receive_clause(not_expected, message)
+  defp do_refute_receive(pattern, timeout, failure_message) do
+    receive_clause = refute_receive_clause(pattern, failure_message)
 
     quote do
       receive do
@@ -621,17 +665,17 @@ defmodule ExUnit.Assertions do
     end
   end
 
- defp refute_receive_clause(not_expected, nil) do
-  binary = Macro.to_string(not_expected)
+ defp refute_receive_clause(pattern, nil) do
+  binary = Macro.to_string(pattern)
   quote do
-    unquote(not_expected) = actual ->
+    unquote(pattern) = actual ->
       flunk "Unexpectedly received message #{inspect actual} (which matched #{unquote binary})"
     end
   end
 
-  defp refute_receive_clause(not_expected, message) do
+  defp refute_receive_clause(pattern, failure_message) do
     quote do
-      unquote(not_expected) -> flunk unquote(message)
+      unquote(pattern) -> flunk unquote(failure_message)
     end
   end
 
