@@ -7,10 +7,10 @@ defmodule Logger do
     * Provides debug, info, warn and error levels.
 
     * Supports multiple backends which are automatically
-      supervised when plugged into Logger.
+      supervised when plugged into `Logger`.
 
     * Formats and truncates messages on the client
-      to avoid clogging Logger backends.
+      to avoid clogging `Logger` backends.
 
     * Alternates between sync and async modes to remain
       performant when required but also apply backpressure
@@ -30,14 +30,14 @@ defmodule Logger do
 
   ## Configuration
 
-  Logger supports a wide range of configurations.
+  `Logger` supports a wide range of configurations.
 
   This configuration is split in three categories:
 
-    * Application configuration - must be set before the Logger
+    * Application configuration - must be set before the `:logger`
       application is started
 
-    * Runtime configuration - can be set before the Logger
+    * Runtime configuration - can be set before the `:logger`
       application is started, but may be changed during runtime
 
     * Error logger configuration - configuration for the
@@ -45,26 +45,33 @@ defmodule Logger do
 
   ### Application configuration
 
-  The following configuration must be set via config files
-  before the Logger application is started.
+  The following configuration must be set via config files (e.g.,
+  `config/config.exs`) before the `:logger` application is started.
 
     * `:backends` - the backends to be used. Defaults to `[:console]`.
       See the "Backends" section for more information.
 
-    * `:compile_time_purge_level` - purge all calls that have log level
-      lower than the configured value at compilation time. This means the
-      Logger call will be completely removed at compile time, accruing
-      no overhead at runtime. Defaults to `:debug` and only
-      applies to the `Logger.debug/2`, `Logger.info/2`, etc style of calls.
+    * `:compile_time_purge_level` - purge *at compilation time* all calls that
+      have log level lower than the value of this option. This means that
+      `Logger` calls with level lower than this option will be completely
+      removed at compile time, accruing no overhead at runtime. Defaults to
+      `:debug` and only applies to the `Logger.debug/2`, `Logger.info/2`,
+      `Logger.warn/2`, and `Logger.error/2` macros (e.g., it doesn't apply to
+      `Logger.log/3`). Note that arguments passed to `Logger` calls that are
+      removed from the AST at compilation time are never evaluated, thus any
+      function call that occurs in these arguments is never executed. As a
+      consequence, avoid code that looks like `Logger.debug("Cleanup:
+      #{perform_cleanup()}")` as in the example `perform_cleanup/0` won't be
+      executed if the `:compile_time_purge_level` is `:info` or higher.
 
     * `:compile_time_application` - sets the `:application` metadata value
       to the configured value at compilation time. This configuration is
       usually only useful for build tools to automatically add the
-      application to the metadata for `Logger.debug/2`, `Logger.info/2`, etc
+      application to the metadata for `Logger.debug/2`, `Logger.info/2`, etc.
       style of calls.
 
   For example, to configure the `:backends` and `compile_time_purge_level`
-  in a `config/config.exs` file:
+  options in a `config/config.exs` file:
 
       config :logger,
         backends: [:console],
@@ -72,25 +79,29 @@ defmodule Logger do
 
   ### Runtime Configuration
 
-  All configuration below can be set via config files but also
-  changed dynamically during runtime via `Logger.configure/1`.
+  All configuration below can be set via config files (e.g.,
+  `config/config.exs`) but also changed dynamically during runtime via
+  `Logger.configure/1`.
 
     * `:level` - the logging level. Attempting to log any message
       with severity less than the configured level will simply
       cause the message to be ignored. Keep in mind that each backend
-      may have its specific level, too.
+      may have its specific level, too. Note that, unlike what happens with the
+      `:compile_time_purge_level` option, the argument passed to `Logger` calls
+      is evaluated even if the level of the call is lower than `:level`.
 
     * `:utc_log` - when `true`, uses UTC in logs. By default it uses
-      local time (i.e. it defaults to `false`).
+      local time (i.e., it defaults to `false`).
 
-    * `:truncate` - the maximum message size to be logged. Defaults
-      to 8192 bytes. Note this configuration is approximate. Truncated
-      messages will have `" (truncated)"` at the end.
+    * `:truncate` - the maximum message size to be logged (in bytes). Defaults
+      to 8192 bytes. Note this configuration is approximate. Truncated messages
+      will have `" (truncated)"` at the end.  The atom `:infinity` can be passed
+      to disable this behavior.
 
-    * `:sync_threshold` - if the Logger manager has more than
-      `sync_threshold` messages in its queue, Logger will change
-      to sync mode, to apply backpressure to the clients.
-      Logger will return to async mode once the number of messages
+    * `:sync_threshold` - if the `Logger` manager has more than
+      `:sync_threshold` messages in its queue, `Logger` will change
+      to *sync mode*, to apply backpressure to the clients.
+      `Logger` will return to *async mode* once the number of messages
       in the queue is reduced to `sync_threshold * 0.75` messages.
       Defaults to 20 messages.
 
@@ -99,7 +110,7 @@ defmodule Logger do
       error reports. This configuration allow developers to change
       how much and how the data should be inspected.
 
-  For example, to configure the `:level` and `:truncate` in a
+  For example, to configure the `:level` and `:truncate` options in a
   `config/config.exs` file:
 
       config :logger,
@@ -108,16 +119,16 @@ defmodule Logger do
 
   ### Error Logger configuration
 
-  The following configuration applies to the Logger wrapper around
+  The following configuration applies to `Logger`'s wrapper around
   Erlang's `error_logger`. All the configurations below must be set
-  before the Logger application starts.
+  before the `:logger` application starts.
 
-    * `:handle_otp_reports` - redirects OTP reports to Logger so
+    * `:handle_otp_reports` - redirects OTP reports to `Logger` so
       they are formatted in Elixir terms. This uninstalls Erlang's
       logger that prints terms to terminal. Defaults to `true`.
 
     * `:handle_sasl_reports` - redirects supervisor, crash and
-      progress reports to Logger so they are formatted in Elixir
+      progress reports to `Logger` so they are formatted in Elixir
       terms. This uninstalls `sasl`'s logger that prints these
       reports to the terminal. Defaults to `false`.
 
@@ -128,14 +139,14 @@ defmodule Logger do
       threshold, the `error_logger` enters discard mode for the
       remainder of that second. Defaults to 500 messages.
 
-  For example, to configure Logger to redirect all `error_logger` messages
+  For example, to configure `Logger` to redirect all `error_logger` messages
   using a `config/config.exs` file:
 
       config :logger,
         handle_otp_reports: true,
         handle_sasl_reports: true
 
-  Furthermore, Logger allows messages sent by Erlang's `error_logger`
+  Furthermore, `Logger` allows messages sent by Erlang's `error_logger`
   to be translated into an Elixir format via translators. Translators
   can be dynamically added at any time with the `add_translator/1`
   and `remove_translator/1` APIs. Check `Logger.Translator` for more
@@ -143,35 +154,40 @@ defmodule Logger do
 
   ## Backends
 
-  Logger supports different backends where log messages are written to.
+  `Logger` supports different backends where log messages are written to.
 
   The available backends by default are:
 
     * `:console` - logs messages to the console (enabled by default)
 
   Developers may also implement their own backends, an option that
-  is explored with detail below.
+  is explored in more detail below.
 
   The initial backends are loaded via the `:backends` configuration,
-  which must be set before the Logger application is started.
+  which must be set before the `:logger` application is started.
 
   ### Console backend
 
-  The console backend logs message to the console. It supports the
-  following options:
+  The console backend logs messages by printing them to the console. It supports
+  the following options:
 
     * `:level` - the level to be logged by this backend.
-      Note that messages are first filtered by the general
-      `:level` configuration in `:logger`
+      Note that messages are filtered by the general
+      `:level` configuration for the `:logger` application first.
 
     * `:format` - the format message used to print logs.
-      Defaults to: `"$time $metadata[$level] $levelpad$message\n"`
+      Defaults to: `"$time $metadata[$level] $levelpad$message\n"`.
 
     * `:metadata` - the metadata to be printed by `$metadata`.
-      Defaults to an empty list (no metadata)
+      Defaults to an empty list (no metadata).
 
     * `:colors` - a keyword list of coloring options.
 
+    * `:max_buffer` - maximum events to buffer while waiting
+      for a confirmation from the IO device (default: 32).
+      Once the buffer is full, the backend will block until
+      a confirmation is received.
+
   In addition to the keys provided by the user via `Logger.metadata/1`,
   the following default keys are available in the `:metadata` list:
 
@@ -211,34 +227,47 @@ defmodule Logger do
 
   ### Custom backends
 
-  Any developer can create their own backend for Logger.
-  Since Logger is an event manager powered by `GenEvent`,
+  Any developer can create their own `Logger` backend.
+  Since `Logger` is an event manager powered by `GenEvent`,
   writing a new backend is a matter of creating an event
   handler, as described in the `GenEvent` module.
 
   From now on, we will be using the term "event handler" to refer
   to your custom backend, as we head into implementation details.
 
-  Once Logger starts, it installs all event handlers under
-  the `:backends` configuration into the Logger event manager.
-  The event manager and all added event handlers are
-  automatically supervised by Logger.
+  Once the `:logger` application starts, it installs all event handlers listed under
+  the `:backends` configuration into the `Logger` event manager. The event
+  manager and all added event handlers are automatically supervised by `Logger`.
 
   Once initialized, the handler should be designed to handle events
   in the following format:
 
-      {level, group_leader,
-        {Logger, message, timestamp, metadata}}
+      {level, group_leader, {Logger, message, timestamp, metadata}} | :flush
 
-  The level is one of `:debug`, `:info`, `:warn` or `:error`,
-  as previously described, the group leader is the group
-  leader of the process which logged the message, followed by
-  a tuple starting with the atom `Logger`, the message as
-  chardata, the timestamp and a keyword list of metadata.
+  where:
+
+    * `level` is one of `:debug`, `:info`, `:warn`, or `:error`, as previously
+      described
+    * `group_leader` is the group leader of the process which logged the message
+    * `{Logger, message, timestamp, metadata}` is a tuple containing information
+      about the logged message:
+      * the first element is always the atom `Logger`
+      * `message` is the actual message (as chardata)
+      * `timestamp` is the timestamp for when the message was logged, as a
+        `{{year, month, day}, {hour, minute, second, milliseconds}}` tuple
+      * `metadata` is a keyword list of metadata used when logging the message
 
   It is recommended that handlers ignore messages where
   the group leader is in a different node than the one where
-  the handler is installed.
+  the handler is installed. For example:
+
+      def handle_event({_level, gl, {Logger, _, _, _}}, state)
+          when node(gl) != node() do
+        {:ok, state}
+      end
+
+  In the case of the event `:flush` handlers should flush any pending data. This
+  event is triggered by `flush/0`.
 
   Furthermore, backends can be configured via the
   `configure_backend/2` function which requires event handlers
@@ -251,11 +280,11 @@ defmodule Logger do
   return value for successful configuration is `:ok`.
 
   It is recommended that backends support at least the following
-  configuration values:
+  configuration options:
 
-    * `level` - the logging level for that backend
-    * `format` - the logging format for that backend
-    * `metadata` - the metadata to include the backend
+    * `:level` - the logging level for that backend
+    * `:format` - the logging format for that backend
+    * `:metadata` - the metadata to include in that backend
 
   Check the implementation for `Logger.Backends.Console`, for
   examples on how to handle the recommendations in this section
@@ -277,14 +306,14 @@ defmodule Logger do
   @doc """
   Adds the given keyword list to the current process metadata.
   """
-  def metadata(dict) do
-    {enabled, metadata} = __metadata__()
+  def metadata(keywords) do
+    {enabled?, metadata} = __metadata__()
     metadata =
-      Enum.reduce(dict, metadata, fn
+      Enum.reduce(keywords, metadata, fn
         {key, nil}, acc -> Keyword.delete(acc, key)
         {key, val}, acc -> Keyword.put(acc, key, val)
       end)
-    Process.put(@metadata, {enabled, metadata})
+    Process.put(@metadata, {enabled?, metadata})
     :ok
   end
 
@@ -295,6 +324,15 @@ defmodule Logger do
     __metadata__() |> elem(1)
   end
 
+  @doc """
+  Resets the current process metadata to the given keyword list.
+  """
+  def reset_metadata(keywords \\ []) do
+    {enabled?, _metadata} = __metadata__()
+    Process.put(@metadata, {enabled?, []})
+    metadata(keywords)
+  end
+
   @doc """
   Enables logging for the current process.
 
@@ -365,8 +403,7 @@ defmodule Logger do
   @spec flush :: :ok
   def flush do
     _ = GenEvent.which_handlers(:error_logger)
-    _ = GenEvent.which_handlers(Logger)
-    :ok
+    GenEvent.sync_notify(Logger, :flush)
   end
 
   @doc """
@@ -380,7 +417,7 @@ defmodule Logger do
 
   """
   def add_backend(backend, opts \\ []) do
-    _ = if opts[:flush], do: GenEvent.which_handlers(:error_logger)
+    _ = if opts[:flush], do: flush()
     case Logger.Watcher.watch(Logger, Logger.Config.translate_backend(backend), backend) do
       {:ok, _} = ok ->
         Logger.Config.add_backend(backend)
@@ -400,7 +437,7 @@ defmodule Logger do
       the backend is removed
   """
   def remove_backend(backend, opts \\ []) do
-    _ = if opts[:flush], do: GenEvent.which_handlers(:error_logger)
+    _ = if opts[:flush], do: flush()
     Logger.Config.remove_backend(backend)
     Logger.Watcher.unwatch(Logger, Logger.Config.translate_backend(backend))
   end
@@ -446,9 +483,11 @@ defmodule Logger do
           level: min_level, utc_log: utc_log?} = Logger.Config.__data__
 
         if compare_levels(level, min_level) != :lt do
+          truncated = truncate(chardata_or_fn, truncate)
           metadata = [pid: self()] ++ Keyword.merge(pdict, metadata)
-          tuple = {Logger, truncate(chardata_or_fn, truncate),
-                   Logger.Utils.timestamp(utc_log?), metadata}
+
+          tuple = {Logger, truncated, Logger.Utils.timestamp(utc_log?), metadata}
+
           try do
             notify(mode, {level, Process.group_leader(), tuple})
             :ok
@@ -512,7 +551,7 @@ defmodule Logger do
 
   @doc """
   Logs a debug message.
-  
+
   Returns the atom `:ok` or an `{:error, reason}` tuple.
 
   ## Examples
@@ -527,7 +566,7 @@ defmodule Logger do
 
   @doc """
   Logs a message.
-  
+
   Returns the atom `:ok` or an `{:error, reason}` tuple.
 
   Developers should use the macros `Logger.debug/2`,
@@ -542,22 +581,29 @@ defmodule Logger do
   defp macro_log(level, data, metadata, caller) do
     %{module: module, function: fun, file: file, line: line} = caller
 
-    caller = [module: module, function: form_fa(fun), file: file, line: line]
-    if app = Application.get_env(:logger, :compile_time_application) do
-      caller = [application: app] ++ caller
-    end
+    caller =
+      compile_time_application ++
+        [module: module, function: form_fa(fun), file: file, line: line]
 
     quote do
       Logger.bare_log(unquote(level), unquote(data), unquote(caller) ++ unquote(metadata))
     end
   end
 
+  defp compile_time_application do
+    if app = Application.get_env(:logger, :compile_time_application) do
+      [application: app]
+    else
+      []
+    end
+  end
+
   defp maybe_log(level, data, metadata, caller) do
     min_level = Application.get_env(:logger, :compile_time_purge_level, :debug)
     if compare_levels(level, min_level) != :lt do
       macro_log(level, data, metadata, caller)
     else
-      :ok
+      handle_unused_variable_warnings(data, caller)
     end
   end
 
@@ -576,4 +622,29 @@ defmodule Logger do
 
   defp notify(:sync, msg),  do: GenEvent.sync_notify(Logger, msg)
   defp notify(:async, msg), do: GenEvent.notify(Logger, msg)
+
+  defp handle_unused_variable_warnings(data, caller) do
+    # We collect all the names of variables (leaving `data` unchanged) with a
+    # scope of `nil` (as we don't warn for variables with a different scope
+    # anyways). We only want the variables that figure in `caller.vars`, as the
+    # AST for calls to local 0-arity functions without parens is the same as the
+    # AST for variables.
+    {^data, logged_vars} = Macro.postwalk(data, [], fn
+      {name, _meta, nil} = var, acc when is_atom(name) ->
+        if {name, nil} in caller.vars, do: {var, [name | acc]}, else: {var, acc}
+      ast, acc ->
+        {ast, acc}
+    end)
+
+    assignments =
+      logged_vars
+      |> Enum.reverse()
+      |> Enum.uniq()
+      |> Enum.map(&quote(do: _ = unquote(Macro.var(&1, nil))))
+
+    quote do
+      unquote_splicing(assignments)
+      :ok
+    end
+  end
 end
