@@ -5,37 +5,34 @@ defmodule Mix.Tasks.DepsTest do
 
   defmodule DepsApp do
     def project do
-      [ app: :deps, version: "0.1.0",
-        deps: [
-          {:ok, "0.1.0",         github: "elixir-lang/ok"},
-          {:invalidvsn, "0.2.0", path: "deps/invalidvsn"},
-          {:invalidapp, "0.1.0", path: "deps/invalidapp"},
-          {:noappfile, "0.1.0",  path: "deps/noappfile"},
-          {:nosemver, "~> 0.1",  path: "deps/nosemver"},
-        ]
-      ]
+      [app: :deps, version: "0.1.0",
+       deps: [
+         {:ok, "0.1.0",         github: "elixir-lang/ok"},
+         {:invalidvsn, "0.2.0", path: "deps/invalidvsn"},
+         {:invalidapp, "0.1.0", path: "deps/invalidapp"},
+         {:noappfile, "0.1.0",  path: "deps/noappfile"},
+         {:nosemver, "~> 0.1",  path: "deps/nosemver"},
+       ]]
     end
   end
 
   defmodule SuccessfulDepsApp do
     def project do
-      [ app: :sample, version: "0.1.0",
-        deps: [
-          {:ok, "0.1.0", path: "deps/ok"}
-        ]
-      ]
+      [app: :sample, version: "0.1.0",
+       deps: [
+         {:ok, "0.1.0", path: "deps/ok"}
+       ]]
     end
   end
 
   defmodule ReqDepsApp do
     def project do
-      [ app: :req_deps, version: "0.1.0",
-        deps: [
-          {:ok, ">= 2.0.0",  path: "deps/ok"},
-          {:noappfile,       path: "deps/noappfile", app: false},
-          {:apppath,         path: "deps/noappfile", app: "../deps/ok/ebin/ok.app"}
-        ]
-      ]
+      [app: :req_deps, version: "0.1.0",
+       deps: [
+         {:ok, ">= 2.0.0",  path: "deps/ok"},
+         {:noappfile,       path: "deps/noappfile", app: false},
+         {:apppath,         path: "deps/noappfile", app: "../deps/ok/ebin/ok.app"}
+       ]]
     end
   end
 
@@ -109,7 +106,7 @@ defmodule Mix.Tasks.DepsTest do
       assert_received {:mix_shell, :info, ["* ok (https://github.com/elixir-lang/ok.git) (mix)"]}
       assert_received {:mix_shell, :info, ["  the dependency is not locked (run \"mix deps.get\" to generate \"mix.lock\" file)"]}
 
-      Mix.Dep.Lock.write %{ok: {:git, "git://github.com/elixir-lang/ok.git", "abcdefghi", []}}
+      Mix.Dep.Lock.write %{ok: {:git, "https://github.com/elixir-lang/ok.git", "abcdefghi", []}}
       Mix.Tasks.Deps.run []
 
       assert_received {:mix_shell, :info, ["* ok (https://github.com/elixir-lang/ok.git) (mix)"]}
@@ -124,13 +121,13 @@ defmodule Mix.Tasks.DepsTest do
     end
   end
 
-  ## deps.check
+  ## deps.loadpaths
 
   test "checks list of dependencies and their status with success" do
     Mix.Project.push SuccessfulDepsApp
 
     in_fixture "deps_status", fn ->
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
     end
   end
 
@@ -139,7 +136,7 @@ defmodule Mix.Tasks.DepsTest do
 
     in_fixture "deps_status", fn ->
       assert_raise Mix.Error, fn ->
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
 
       assert_received {:mix_shell, :error, ["* ok (https://github.com/elixir-lang/ok.git)"]}
@@ -163,28 +160,32 @@ defmodule Mix.Tasks.DepsTest do
       File.rm_rf("_build")
 
       Mix.Tasks.Deps.Compile.run []
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
       assert File.exists?("_build/dev/lib/ok/ebin/ok.app")
       assert File.exists?("_build/dev/lib/ok/priv/sample")
 
       Mix.Tasks.Compile.run []
+      assert to_charlist(Path.expand("_build/dev/lib/ok/ebin/")) in :code.get_path
       assert File.exists?("_build/dev/lib/sample/ebin/sample.app")
 
-      # Remove the deps but set build_path, deps won't be pruned
+      # Remove the deps but set build_path, deps won't be pruned, but load paths are
       Mix.ProjectStack.post_config [deps: [], build_path: "_build"]
+      Mix.ProjectStack.clear_cache
       Mix.Project.pop
       Mix.Project.push SuccessfulDepsApp
 
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
+      refute to_charlist(Path.expand("_build/dev/lib/ok/ebin/")) in :code.get_path
       assert File.exists?("_build/dev/lib/ok/ebin/ok.app")
       assert File.exists?("_build/dev/lib/sample/ebin/sample.app")
 
       # Remove the deps without build_path, deps will be pruned
       Mix.ProjectStack.post_config [deps: []]
+      Mix.ProjectStack.clear_cache
       Mix.Project.pop
       Mix.Project.push SuccessfulDepsApp
 
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
       refute File.exists?("_build/dev/lib/ok/ebin/ok.app")
       assert File.exists?("_build/dev/lib/sample/ebin/sample.app")
     end
@@ -192,6 +193,20 @@ defmodule Mix.Tasks.DepsTest do
 
   ## deps.unlock
 
+  test "cleans and recompiles artifacts if --force given" do
+    Mix.Project.push SuccessfulDepsApp
+
+    in_fixture "deps_status", fn ->
+      Mix.Tasks.Deps.Compile.run []
+      File.touch! "_build/dev/lib/ok/clean-me"
+
+      Mix.Tasks.Deps.Compile.run ["--force"]
+      refute File.exists? "_build/dev/lib/ok/clean-me"
+    end
+  end
+
+  ## deps.unlock
+
   test "unlocks all deps", context do
     Mix.Project.push DepsApp
     in_tmp context.test, fn ->
@@ -223,6 +238,21 @@ defmodule Mix.Tasks.DepsTest do
     end
   end
 
+  test "unlocks filtered deps", context do
+    Mix.Project.push DepsApp
+    in_tmp context.test, fn ->
+      Mix.Dep.Lock.write %{git_repo: "abcdef", another: "hash", another_one: "hash"}
+      Mix.Tasks.Deps.Unlock.run ["--filter", "another"]
+      assert Mix.Dep.Lock.read == %{git_repo: "abcdef"}
+      output = """
+      Unlocked deps:
+      * another
+      * another_one
+      """
+      assert_received {:mix_shell, :info, [^output]}
+    end
+  end
+
   ## Deps environment
 
   defmodule DepsEnvApp do
@@ -308,7 +338,7 @@ defmodule Mix.Tasks.DepsTest do
     end
   end
 
-  defmodule OverridenDepsApp do
+  defmodule OverriddenDepsApp do
     def project do
       [
         app: :raw_sample,
@@ -321,7 +351,7 @@ defmodule Mix.Tasks.DepsTest do
     end
   end
 
-  defmodule NonOverridenDepsApp do
+  defmodule NonOverriddenDepsApp do
     def project do
       [
         app: :raw_sample,
@@ -349,7 +379,7 @@ defmodule Mix.Tasks.DepsTest do
 
     in_fixture "deps_status", fn ->
       assert_raise Mix.Error, fn ->
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
       assert_received {:mix_shell, :error, ["  the dependency git_repo in mix.exs is overriding a child dependency" <> _]}
 
@@ -370,12 +400,12 @@ defmodule Mix.Tasks.DepsTest do
 
     in_fixture "deps_status", fn ->
       assert_raise Mix.Error, fn ->
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
 
       assert_received {:mix_shell, :error, ["  different specs were given for the git_repo app:" <> _ = msg]}
       assert msg =~ "In custom/deps_repo/mix.exs:"
-      assert msg =~ "{:git_repo, \"0.1.0\", [git: #{inspect fixture_path("git_repo")}]}"
+      assert msg =~ "{:git_repo, \"0.1.0\", [env: :prod, git: #{inspect fixture_path("git_repo")}]}"
     end
   end
 
@@ -401,12 +431,12 @@ defmodule Mix.Tasks.DepsTest do
 
       assert_raise Mix.Error, fn ->
         Mix.Tasks.Deps.Get.run []
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
 
-      assert_received {:mix_shell, :error, ["  the dependency git_repo" <> _ = msg]}
+      assert_received {:mix_shell, :error, ["  the dependency git_repo 0.1.0" <> _ = msg]}
       assert msg =~ "In custom/deps_repo/mix.exs:"
-      assert msg =~ "{:git_repo, \"0.2.0\", [git: #{inspect fixture_path("git_repo")}]}"
+      assert msg =~ "{:git_repo, \"0.2.0\", [env: :prod, git: #{inspect fixture_path("git_repo")}]}"
     end
   end
 
@@ -432,7 +462,7 @@ defmodule Mix.Tasks.DepsTest do
 
       assert_raise Mix.Error, fn ->
         Mix.Tasks.Deps.Get.run []
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
 
       assert_received {:mix_shell, :error, ["  the dependency git_repo in mix.exs is overriding" <> _]}
@@ -462,7 +492,7 @@ defmodule Mix.Tasks.DepsTest do
   end
 
   test "works with overridden dependencies" do
-    Mix.Project.push OverridenDepsApp
+    Mix.Project.push OverriddenDepsApp
 
     in_fixture "deps_status", fn ->
       Mix.Tasks.Deps.Get.run []
@@ -484,17 +514,17 @@ defmodule Mix.Tasks.DepsTest do
   end
 
   test "converged dependencies errors if not overriding" do
-    Mix.Project.push NonOverridenDepsApp
+    Mix.Project.push NonOverriddenDepsApp
 
     in_fixture "deps_status", fn ->
       assert_raise Mix.Error, fn ->
-        Mix.Tasks.Deps.Check.run []
+        Mix.Tasks.Deps.Loadpaths.run []
       end
 
       receive do
         {:mix_shell, :error, ["  the dependency git_repo in mix.exs" <> _ = msg]} ->
           assert msg =~ "In mix.exs:"
-          assert msg =~ "{:git_repo, \"0.1.0\", [git: #{inspect fixture_path("git_repo")}]}"
+          assert msg =~ "{:git_repo, \"0.1.0\", [env: :prod, git: #{inspect fixture_path("git_repo")}]}"
       after
         0 -> flunk "expected overriding error message"
       end
@@ -508,10 +538,11 @@ defmodule Mix.Tasks.DepsTest do
 
     in_fixture "deps_status", fn ->
       Mix.Tasks.Deps.Compile.run []
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
 
       File.mkdir_p!("_build/dev/lib/ok/ebin")
-      File.write!("_build/dev/lib/ok/.compile.elixir_scm", ~s({v1, <<\"the_future\">>, scm}.))
+      manifest_data = :erlang.term_to_binary({:v1, "the_future", :scm})
+      File.write!("_build/dev/lib/ok/.compile.elixir_scm", manifest_data)
       Mix.Task.clear
 
       msg = "  the dependency was built with an out-of-date Elixir version, run \"mix deps.compile\""
@@ -519,8 +550,8 @@ defmodule Mix.Tasks.DepsTest do
       Mix.Tasks.Deps.run []
       assert_received {:mix_shell, :info, [^msg]}
 
-      # deps.check will automatically recompile it
-      Mix.Tasks.Deps.Check.run []
+      # deps.loadpaths will automatically recompile it
+      Mix.Tasks.Deps.Loadpaths.run []
 
       Mix.Tasks.Deps.run []
       refute_received {:mix_shell, :info, [^msg]}
@@ -532,10 +563,11 @@ defmodule Mix.Tasks.DepsTest do
 
     in_fixture "deps_status", fn ->
       Mix.Tasks.Deps.Compile.run []
-      Mix.Tasks.Deps.Check.run []
+      Mix.Tasks.Deps.Loadpaths.run []
 
       File.mkdir_p!("_build/dev/lib/ok/ebin")
-      File.write!("_build/dev/lib/ok/.compile.elixir_scm", ~s({v1, <<"#{System.version}">>, scm}.))
+      manifest_data = :erlang.term_to_binary({:v1, System.version, :scm})
+      File.write!("_build/dev/lib/ok/.compile.elixir_scm", manifest_data)
       Mix.Task.clear
 
       msg = "  the dependency was built with another SCM, run \"mix deps.compile\""
@@ -543,8 +575,8 @@ defmodule Mix.Tasks.DepsTest do
       Mix.Tasks.Deps.run []
       assert_received {:mix_shell, :info, [^msg]}
 
-      # deps.check will automatically recompile it
-      Mix.Tasks.Deps.Check.run []
+      # deps.loadpaths will automatically recompile it
+      Mix.Tasks.Deps.Loadpaths.run []
 
       Mix.Tasks.Deps.run []
       refute_received {:mix_shell, :info, [^msg]}
@@ -606,7 +638,8 @@ defmodule Mix.Tasks.DepsTest do
         app: :raw_sample,
         version: "0.1.0",
         deps: [
-          {:git_repo, ">= 0.1.0", git: MixTest.Case.fixture_path("git_repo")}
+          {:git_repo, ">= 0.1.0", git: MixTest.Case.fixture_path("git_repo")},
+          {:ok, ">= 2.0.0",  path: "deps/ok"}
         ]
       ]
     end
@@ -619,6 +652,8 @@ defmodule Mix.Tasks.DepsTest do
       File.mkdir_p!("_build/dev/lib/raw_sample")
       File.mkdir_p!("_build/dev/lib/git_repo")
       File.mkdir_p!("_build/test/lib/git_repo")
+      File.mkdir_p!("_build/dev/lib/ok")
+      File.mkdir_p!("_build/test/lib/ok")
 
       message = "\"mix deps.clean\" expects dependencies as arguments or " <>
                 "a flag indicating which dependencies to clean. " <>
@@ -631,6 +666,7 @@ defmodule Mix.Tasks.DepsTest do
 
       Mix.Tasks.Deps.Clean.run ["--only", "dev", "--all"]
       refute File.exists?("_build/dev/lib/git_repo")
+      refute File.exists?("_build/dev/lib/ok")
       assert File.exists?("_build/test/lib/git_repo")
       assert File.exists?("_build/dev/lib/raw_sample")
 
@@ -672,4 +708,17 @@ defmodule Mix.Tasks.DepsTest do
       refute File.exists?("_build/dev/lib/raw_sample")
     end
   end
+
+  test "does not remove dependency source when using :path" do
+    Mix.Project.push CleanDepsApp
+
+    in_fixture "deps_status", fn ->
+      assert File.exists?("deps/ok")
+
+      Mix.Tasks.Deps.Clean.run ["raw_sample", "--all"]
+      refute File.exists?("_build/dev/lib/ok")
+      refute File.exists?("_build/test/lib/ok")
+      assert File.exists?("deps/ok")
+    end
+  end
 end
